//==============================================
//  copyright            : (C) 2003 by Will Stokes
//==============================================
//  This program is free software; you can redistribute it 
//  and/or modify it under the terms of the GNU General 
//  Public License as published by the Free Software 
//  Foundation; either version 2 of the License, or  
//  (at your option) any later version.         
//
//  As a special exception, Will Stokes gives permission to 
//  link this program with Qt non-commercial edition, and 
//  distribute the resulting executable, without including the 
//  source code for the Qt non-commercial edition in the 
//  source distribution. 
//==============================================

//Systemwide includes
#include <qimage.h>
#include <qpixmap.h>
#include <qstring.h>
#include <qtextstream.h>
#include <qdom.h>
#include <fstream.h>
#include <qdir.h>
#include <qregexp.h>

//Projectwide includes
#include "album.h"
#include "subalbum.h"
#include "photo.h"
#include "imageTools.h"
#include "xmlTools.h"
#include "md5.h"
#include "../gui/photoWidget.h"
#include "../gui/loadDialog.h"
#include "../config.h"

#include <iostream.h>

//==============================================
///Sets default information
Subalbum::Subalbum(Album* albm, int number)
{
  //set subalbum number
  this->number = number;

  //by default no photos in subalbum
  numPhotos = 0;
  loadedPhotos = 0;
  
  //set strings to default values
  name = "";
  description ="";

  //set default rep images
  smallRepresentativeImage = new QPixmap( QString(IMAGE_PATH)+"notSpecified.png" );
  mediumRepresentativeImage = new QPixmap( QString(IMAGE_PATH)+"subalbum.png" );
  largeRepresentativeImage = NULL;

  //no photos by default
  firstPhoto = NULL;
  lastPhoto = NULL;
  
  //next pointer null by default
  nextSubalbum = NULL;
  
  //set album pointer
  this->albm = albm;
}
//==============================================
///Frees photos
Subalbum::~Subalbum()
{
  //delete representative images
  delete smallRepresentativeImage;
  delete mediumRepresentativeImage;
  delete largeRepresentativeImage;
  
  //delete all photos
  Photo* current = firstPhoto;
  Photo* temp;
  while(current != NULL)
  {
    temp = current->getNext();
    delete current;
    current = temp;
  }
} 
//==============================================
///Sets the Subalbum Name
void Subalbum::setName(QString val)
{
  name = val;
}
//==============================================
///Gets the Subalbum Name
QString Subalbum::getName()
{
  return QString(name);
}
//==============================================
///Sets the Subalbum description
void Subalbum::setDescription(QString val)
{
  description = val;
}
//==============================================
///Gets the Subalbum description
QString Subalbum::getDescription()
{
  return QString(description);
}
//==============================================
//gets a sized representative image  
QPixmap* Subalbum::getRepresentativeImage(int size)
{
  if(size == SMALL)
    return smallRepresentativeImage;
  if(size == MEDIUM)
    return mediumRepresentativeImage;
  if(size == LARGE)
    return largeRepresentativeImage;
  else
    return NULL;
}
//==============================================
//sets a sized representative iamge
void Subalbum::setRepresentativeImages(QImage* rawThumbnail)
{
  //---------------------------------------------------------    
  //delete old representative images
  delete smallRepresentativeImage;
  delete mediumRepresentativeImage;
  delete largeRepresentativeImage;
  //---------------------------------------------------------    
  //compute various representative image sizes
  int smallRepWidth = 0;
  int smallRepHeight = 0;
  int mediumRepWidth = 0;
  int mediumRepHeight = 0;  
  resizeImage( rawThumbnail->width(), rawThumbnail->height(),
               131, 98,
               mediumRepWidth, mediumRepHeight);
  resizeImage( rawThumbnail->width(), rawThumbnail->height(),
               107, 80,
               smallRepWidth, smallRepHeight);
  //---------------------------------------------------------    
  //create various representative images
  //---------------------------------------------------------    
  //copy and scale small version
  QImage thumbnailSmall = rawThumbnail->smoothScale( smallRepWidth, smallRepHeight );
  smallRepresentativeImage = new QPixmap( smallRepWidth, smallRepHeight );
  smallRepresentativeImage->convertFromImage( thumbnailSmall );  
  
  //copy and scale medium version
  QImage thumbnailMedium = rawThumbnail->smoothScale( mediumRepWidth, mediumRepHeight );
  QImage* centeredThumbnailMedium = new QImage(131, 98, thumbnailMedium.depth());
  centeredThumbnailMedium->setAlphaBuffer(true);
 
  int xDiff = 131 - mediumRepWidth;
  int yDiff = 98  - mediumRepHeight;
  
  //set all pixels to white
  int x, y;
  for(x=0; x< 131; x++)
  {
    for(y=0; y<98; y++)
    {
      centeredThumbnailMedium->setPixel(x, y, QColor(255, 255, 255).rgb()); 
    }
  }
  
  int x2 = 0;
  for(x= xDiff/2; x < (xDiff/2) + mediumRepWidth; x++)
  {
    int y2 = 0;
    for(y= yDiff/2; y < (yDiff/2) + mediumRepHeight; y++)
    {
      centeredThumbnailMedium->setPixel(x, y, thumbnailMedium.pixel(x2, y2));
      y2++;
    }
    x2++;
  }
  
  mediumRepresentativeImage = new QPixmap( centeredThumbnailMedium->width(), centeredThumbnailMedium->height() );
  mediumRepresentativeImage->convertFromImage( *centeredThumbnailMedium );
  delete centeredThumbnailMedium;
  //copy large version 
  largeRepresentativeImage = new QPixmap(*rawThumbnail);
  //---------------------------------------------------------    
}
//==============================================
///Adds a new photo to the Subalbum and appends it to the end
bool Subalbum::addPhoto(QString fileName, Photo* newPhoto)
{
  numPhotos++;
  
  //create new photo if necessary
  if(newPhoto == NULL)
    newPhoto = new Photo(this, numPhotos);
    
  QString desc(fileName);  
  desc = desc.section( QRegExp("/"), -1);
  desc.truncate(desc.length() - 4);
  newPhoto->setDescription( desc );
  
  //attempt to set image
  if(!newPhoto->setImage(fileName))
  {
    cout << "Error! Unable to load " << fileName << endl;
    delete newPhoto;
    return false;
  }
  
  //if this is the only photo, set first and last 
  //pointers to this photo.
  if(firstPhoto == NULL)
  {
    firstPhoto = newPhoto;
    lastPhoto = newPhoto;
  }
  //else append to end of list
  else
  {
    lastPhoto->setNext(newPhoto);
    lastPhoto = newPhoto;
  }
  
  //copy image file over
  QString saveName = QDir::homeDirPath() + QString("/.albumShaper/tmp/%1_%2")
                                .arg(number)
                                .arg(numPhotos);
  
  newPhoto->getImage(IMAGE)->save( saveName + ".jpg", "JPEG", 100);
  newPhoto->getImage(SLIDESHOW)->save( saveName + "_slideshow.jpg", "JPEG", 100);
  
  newPhoto->setImageFilename(saveName + ".jpg");
  newPhoto->setSlideshowFilename(saveName + "_slideshow.jpg");
  
  //deallocate full image and slideshow versions to conserve memory
  newPhoto->deallocateLargeImages();
  
  return true;
}
//==============================================
bool Subalbum::lazyAddPhoto(QString imageName, 
                                          QString slideshowName, 
                                          QString thumbnailName, 
                                          Photo* newPhoto)
{
  numPhotos++;
  
  //attempt to set image
  if(!newPhoto->setImage(imageName, slideshowName, thumbnailName))
  {
    cout << "Error! Unable to load " << imageName << endl;
    delete newPhoto;
    return false;
  }
  
  //if this is the only photo, set first and last 
  //pointers to this photo.
  if(firstPhoto == NULL)
  {
    firstPhoto = newPhoto;
    lastPhoto = newPhoto;
  }
  //else append to end of list
  else
  {
    lastPhoto->setNext(newPhoto);
    lastPhoto = newPhoto;
  }
  
  return true;
}
//==============================================
///Removes a specified photo
void Subalbum::removePhoto(Photo* val)
{ 
  //walk through list of photos and find specified photo
  Photo* current = firstPhoto;
  Photo* prev = NULL;
  while(current != val && current->getNext() != NULL)
  {
    prev = current;
    current = current->getNext();
  }

  if(current == val)
  {
    //update prev next pointer
    if(prev != NULL)
      prev->setNext(current->getNext());
    
    //update first and last pointers if necessary
    if(current == firstPhoto)
      firstPhoto = current->getNext();
    if(current == lastPhoto)
      lastPhoto = prev;

    //free Photo
    delete current;
    current = NULL;
    numPhotos--;
  }
}
//==============================================
Subalbum* Subalbum::getNext()
{
  return nextSubalbum;
}
//==============================================
void Subalbum::setNext(Subalbum* val)
{
  nextSubalbum = val;
}
//==============================================
void Subalbum::exportToXML(QTextStream& stream)
{
  //write subalbum information
  stream << "  <subalbum>\n";
  stream << "    <name>" << fixXMLString(name) << "</name>\n";
  stream << "    <description>" << fixXMLString(description) << "</description>\n";
  
  //write photos
  Photo* current = firstPhoto;
  while(current != NULL)
  {
    current->exportToXML(stream);
    current = current->getNext();
  }  
  
  //close subalbum
  stream << "  </subalbum>\n";

}
//==============================================
void Subalbum::exportThumbnailHTML(QTextStream& stream, int subalbumNumber)
{
  stream << "<!DOCTYPE html PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">\n";    
  stream << "<html>\n";
  stream << "  <head>\n";
  stream << "    <style type=\"text/html\">\n";
  stream << "      <!--\n";
  stream << "      A{text-decoration:none}\n";
  stream << "      -->\n";
  stream << "    </style>\n";
  stream << "    <style type=\"text/css\">\n";
  stream << "     <!--\n";
  stream << "     h3 {font-size: 12pt; font-weight: bold; text-align: center}\n";
  stream << "     h4 {font-size: 10pt; font-weight: normal; text-align: center}\n";
  stream << "     h4.ital {font-size: 10pt; font-weight: normal; text-align: center}\n";
  stream << "      -->\n";
  stream << "    </style>\n";    
  stream << "    <title>" << name << "</title>\n";
  stream << "    <meta name=\"generator\" ";
  stream << "content=\"Album Shaper (c.) Will Stokes\">\n";
  stream << "  </head>\n";
  stream << "  <body>\n";
  stream << "    <center>\n";
  stream << "      <table>\n";
  stream << "        <tr>\n";
  stream << "          <td width=\"450\">\n";
  stream << "            <h3>\n";
  stream << "              " << name << "\n";
  stream << "           </h3>\n";
  stream << "           <h4>\n";
  stream << "             " << description << "\n";
  stream << "           </h4>\n";
  stream << "          </td>\n";
  stream << "        </tr>\n";
  stream << "      </table>\n";
  stream << "    </center>\n";
  stream << "    <h4>\n";
  stream << "    <script type=\"text/javascript\" language=\"JavaScript\">\n";
  stream << "    <!-- HIDE FROM OLD BROWSERS\n";
  stream << "    document.write(\"<a href=\\\"subalbum_" << subalbumNumber << "_slideshow.html\\\">Begin Slide Show</a>\")\n";
  stream << "    document.write(\"&nbsp;-&nbsp;\")\n";
  stream << "    -->\n";
  stream << "    </script>\n";
  stream << "      <a href=\"Album.html\">\n";
  stream << "        Album Index\n";
  stream << "      </a>\n";
  stream << "    </h4>\n";
    
  int photoNumber = 0;
    
  //write photo thumbnails
  Photo* current = firstPhoto;
  if(current != NULL)
  {
    stream << "    <center>\n";
    stream << "      <table border=\"0\">\n";
    while(current != NULL)
    {
      Photo* backupCurrent = current;
      int backupNumber = photoNumber;
      //---------------------------------------
      //Print row of images
      //---------------------------------------
      int n = 3;
      stream << "        <tr>\n";
      while(current != NULL && n > 0)
      {
        n--;
        photoNumber++;
        stream << "          <td valign=\"bottom\" width=\"220\">\n";
        stream << "           <center>\n";
        stream << "            <a href=\"img/" << subalbumNumber << "/" << photoNumber << ".jpg\">\n";
        stream << "             <img alt=\"" << current->getDescription() << "\" src=\"img/" << subalbumNumber << "/" << photoNumber << "_thumb.jpg\">\n";
        stream << "            </a>\n";
        stream << "           </center>\n";
        stream << "          </td>\n";
        current = current->getNext();
      }
      
      while(n != 0 && photoNumber > 2)
      {
        stream << "          <td width=\"220\"></td>\n";
        n--;
      }
      stream << "        </tr>\n";
      //---------------------------------------
      //Print row of image descriptions
      //---------------------------------------
      current = backupCurrent;
      photoNumber = backupNumber;
      n = 3;
      stream << "        <tr>\n";
      while(current != NULL && n > 0)
      {
        n--;
        photoNumber++;
        stream << "          <td valign=\"top\" width=\"220\">\n";
        stream << "           <center>\n";
        stream << "             <a href=\"img/" << subalbumNumber << "/" << photoNumber << ".jpg\">\n";
        stream << "               " << current->getDescription() << "\n";
        stream << "            </a>\n";
        stream << "           </center>\n";
        stream << "          </td>\n";
        current = current->getNext();
      }
      
      while(n != 0 && photoNumber > 2)
      {
        stream << "          <td width=\"220\"></td>\n";
        n--;
      }
      stream << "        </tr>\n";
    }
      
    stream << "      </table>\n";
    stream << "    </center>\n";
  } //end if
  
  stream << "    <h4 class=\"ital\">\n";
  stream << "      (Generated by <a href=\"http://albumshaper.sourceforge.net\">Album Shaper</a>)\n";
  stream << "    </h4>\n";
  stream << "  </body>\n";
  stream << "</html>\n";
}
//==============================================
void Subalbum::exportSlideshowHTML(QTextStream& stream, int subalbumNumber)
{
  stream << "<!DOCTYPE html PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">\n";    
  stream << "<html>\n";
  stream << "  <head>\n";
  stream << "    <style type=\"text/html\">\n";
  stream << "      <!--\n";
  stream << "      A{text-decoration:none}\n";
  stream << "      -->\n";
  stream << "    </style>\n";
  stream << "    <style type=\"text/css\">\n";
  stream << "     <!--\n";
  stream << "     h3 {font-size: 12pt; font-weight: bold; text-align: center}\n";
  stream << "     h4 {font-size: 10pt; font-weight: normal; text-align: center}\n";
  stream << "     h4.ital {font-size: 10pt; font-weight: normal; text-align: center}\n";
  stream << "      -->\n";
  stream << "    </style>\n";    
  stream << "    <title>" << name << "</title>\n";
  stream << "    <meta name=\"generator\" content=\"Album Shaper (c.) Will Stokes\">\n";
  stream << "    <script type=\"text/javascript\" language=\"JavaScript\">\n"; 
  stream << "    <!-- HIDE FROM OLD BROWSERS\n";
  stream << "    var imageNumber=1\n\n"; 
  stream << "    var photoTitles = new Array(" << numPhotos << ");\n";
  
  Photo* current = firstPhoto;
  int n = 0;
  while(current != NULL)
  {
    stream << "    photoTitles[" << n << "] = \"" << current->getDescription() << "\";\n";
    n++;
    current = current->getNext();
  }
  
  stream << "    //###################################### \n";
  stream << "    //Define functions used to change images \n";
  stream << "    //###################################### \n";
  stream << "    function expandImage() \n";
  stream << "    { \n";
  stream << "      fileToOpen = \"img/" << subalbumNumber << "/\"+imageNumber+\".jpg\" \n";
  stream << "      new_window = window.open(fileToOpen,'albulmifyRules') \n";
  stream << "    } \n\n";
  stream << "    function reloadImage() \n";
  stream << "    { \n";
  stream << "      document.currentPhoto.src=\"img/" << subalbumNumber << "/\"+imageNumber+\"_slideshow.jpg\"\n";
  stream << "      document.currentPhoto.alt=photoTitles[imageNumber-1];\n";
  stream << "      document.photoInfo.whichPhoto.value=\"Photo \" + imageNumber + \" of " << numPhotos << "\"\n";
  stream << "      document.photoInfo.photoDesc.value=photoTitles[imageNumber-1];\n";
  stream << "    } \n\n";
  stream << "    function firstImage() \n";
  stream << "    { \n";
  stream << "      //set image number to first \n";
  stream << "      imageNumber = 1 \n\n";
  stream << "      //reset image src value based on variable \n";
  stream << "      reloadImage() \n";
  stream << "    } \n\n";
  stream << "    function lastImage() \n";
  stream << "    { \n";
  stream << "      //set image number to last \n";
  stream << "      imageNumber = " << numPhotos << "\n\n";
  stream << "      //reset image src value based on variable \n";
  stream << "      reloadImage() \n";
  stream << "    } \n\n";
  stream << "    function nextImage() \n";
  stream << "    { \n";
  stream << "      //increment image number \n";
  stream << "      imageNumber = imageNumber + 1 \n\n";
  stream << "      //wrap to begining if reached the end \n";
  stream << "      if (imageNumber==" << (numPhotos+1) << ") \n";
  stream << "      {imageNumber = 1} \n\n";
  stream << "      //reset image src value based on variable \n";
  stream << "      reloadImage() \n";
  stream << "    } \n\n";
  stream << "    function previousImage() \n";
  stream << "    { \n";
  stream << "      //decrement image number \n";
  stream << "      imageNumber = imageNumber - 1 \n";
  stream << "      //wrap to begining if reached the end \n";
  stream << "      if (imageNumber==(1 - 1)) \n";
  stream << "      {imageNumber = " << numPhotos << "} \n\n";
  stream << "      //reset image src value based on variable \n";
  stream << "      reloadImage() \n";
  stream << "    }\n\n";
  stream << "   -->\n";
  stream << "   </script> \n";
  stream << "  </head> \n";
  stream << "  <body> \n";
  stream << "    <center> \n";
  stream << "      <table>\n";
  stream << "        <tr>\n";
  stream << "          <td width=\"450\">\n";
  stream << "            <h3>\n";
  stream << "              " << name << "\n";
  stream << "           </h3>\n";
  stream << "           <h4>\n";
  stream << "             " << description << "\n";
  stream << "           </h4>\n";
  stream << "          </td>\n";
  stream << "        </tr>\n";
  stream << "      </table>\n";
  stream << "    </center> \n";
  stream << "    <h4>\n";
  stream << "      <a href=\"subalbum_" << subalbumNumber << "_thumbs.html\">\n";
  stream << "        Thumbnails\n";
  stream << "      </a>\n";
  stream << "      &nbsp;-&nbsp;\n";
  stream << "      <a href=\"Album.html\">\n";
  stream << "        Album Index\n";
  stream << "      </a>\n";
  stream << "    </h4>\n";
  stream << "    <center> \n";
  stream << "      <a href=\"JavaScript:expandImage()\"> \n";
  stream << "        <img alt=\"current photo\" src=\"img/" << subalbumNumber << "/1_slideshow.jpg\" name=\"currentPhoto\" border=\"0\"> \n";
  stream << "      </a> \n";
  stream << "    </center> \n";
  stream << "    <center> \n";
  stream << "      <form name=\"photoInfo\" action=\"\"> \n";
  stream << "        <table border=\"0\"> \n";
  stream << "          <tr> \n";
  stream << "            <td valign=\"middle\" align=\"center\" width=\"100\"> \n";
  stream << "              <a href=\"JavaScript:firstImage()\"> (First) </a> \n";
  stream << "            </td> \n";
  stream << "             <td valign=\"middle\" align=\"center\" width=\"100\"> \n";
  stream << "               <a href=\"JavaScript:previousImage()\"> Previous </a> \n";
  stream << "            </td> \n";
  stream << "            <td valign=\"middle\" align=\"center\">\n";
  stream << "              <input type=\"text\" maxlength=\"100\" name=\"whichPhoto\" readonly value=\"Photo 1 of " << numPhotos << "\">\n";
  stream << "            </td>\n";
  stream << "            <td valign=\"middle\" align=\"center\" width=\"100\"> \n";
  stream << "              <a href=\"JavaScript:nextImage()\"> Next </a> \n";
  stream << "            </td> \n";
  stream << "            <td valign=\"middle\" align=\"center\" width=\"100\"> \n";
  stream << "              <a href=\"JavaScript:lastImage()\"> (Last) </a> \n";
  stream << "            </td> \n";
  stream << "          </tr> \n";
  stream << "          <tr> \n";  
  stream << "            <td colspan=\"5\" valign=\"middle\" align=\"center\"> \n";
  stream << "              <textarea cols=\"40\" rows=\"2\" name=\"photoDesc\" readonly>\n";
  if( firstPhoto == NULL || firstPhoto->getDescription() == "" )
  {
    stream << "Photo 1 of " << numPhotos << "\n";
  }
  else
  {
    stream << firstPhoto->getDescription() << "\n";
  }
  stream << "              </textarea>\n";
  stream << "            </td> \n";
  stream << "          </tr> \n";
  stream << "        </table> \n";
  stream << "      </form> \n";
  stream << "    </center> \n";
  stream << "    <h4 class=\"ital\">\n";
  stream << "      (Generated by <a href=\"http://albumshaper.sourceforge.net\">Album Shaper</a>)\n";
  stream << "    </h4>\n";
  stream << "  </body> \n";
  stream << "</html> \n";
}
//==============================================
Photo* Subalbum::getFirst()
{
  return firstPhoto;
}
//==============================================
Photo* Subalbum::getLast()
{
  return lastPhoto;
}
//==============================================
void Subalbum::importFromDisk(QDomNode* root, int subalbumNum, LoadDialog* dialog, QString dirName)
{
  //if representative image exists load
  QString repName = QString(dirName + "/img/%1_thumb.jpg").arg(subalbumNum);
  QImage repImage(repName);
  if(!repImage.isNull())
  {
    setRepresentativeImages(&repImage);  
  }  
  
  QDomNode node = root->firstChild();
  QDomText val;  
  int photoNum = 0;
  while( !node.isNull() )
  {
    //------------------------------------------------------------
    //subalbum name
    if( node.isElement() && node.nodeName() == "name" )
    { 
      val = node.firstChild().toText();
      if(!val.isNull())
        name = val.nodeValue();
    }
    //------------------------------------------------------------
    //subalbum description
    else if( node.isElement() && node.nodeName() == "description" )
    { 
      val = node.firstChild().toText();
      if(!val.isNull())
        description = val.nodeValue();
    }
    //------------------------------------------------------------
    //photo
    else if( node.isElement() && node.nodeName() == "photo" )
    { 
      //increase counter
      photoNum++;
      

      dialog->printSubalbumPhoto(subalbumNum, photoNum);
      
      //create new photo object
      QString imageName = QString(dirName + "img/%1/%2.jpg").arg(subalbumNum).arg(photoNum);
      QString slideshowName = QString(dirName + "img/%1/%2_slideshow.jpg").arg(subalbumNum).arg(photoNum);
      QString thumbName = QString(dirName + "img/%1/%2_thumb.jpg").arg(subalbumNum).arg(photoNum);
      Photo* newPhoto = new Photo(this, photoNum);

      //load photo information from disk
      newPhoto->importFromDisk( &node );      
      
      //if no changes have occured do lazy load - don't 
      //bother scaling down thumbnail and slideshow images
      //from original image
      ifstream imageFile( imageName );      
      ifstream slideshowFile( slideshowName );      
      ifstream thumbnailFile( thumbName );
      
      if(
          imageFile.is_open() &&
          thumbnailFile.is_open() &&
          slideshowFile.is_open() &&
          getMD5(imageFile) == newPhoto->getImageChecksum() &&
          getMD5(slideshowFile) == newPhoto->getSlideshowChecksum() &&
          getMD5(thumbnailFile) == newPhoto->getThumbnailChecksum()
        )
      {
        //close ifstreams
        imageFile.close();
        slideshowFile.close();
        thumbnailFile.close();
        
        //populate image
        lazyAddPhoto(imageName, slideshowName, thumbName, newPhoto);
      }
      //else reload image and scale it since changes have occured.
      else
      {
        //close ifstreams if open
        if(imageFile.is_open())
          imageFile.close();
        if(thumbnailFile.is_open())
          thumbnailFile.close();
          
        //populate image
        addPhoto(imageName, newPhoto);
      }

      if(imageFile.is_open())
        imageFile.close();
      if(slideshowFile.is_open())
        slideshowFile.close();
      if(thumbnailFile.is_open())
        thumbnailFile.close();
    }
    //------------------------------------------------------------
    //advance to next node   
    node = node.nextSibling();
    //------------------------------------------------------------
  }
  //------------------------------------------------------------
  //set loaded number
  resetNumLoadedPhotos();
  //------------------------------------------------------------  
}
//==============================================
void Subalbum::addPhoto(Photo* newPhoto)
{
  //increase counter
  numPhotos++;
  
  //set it's next pointer to null
  newPhoto->setNext(NULL);

  //if this is the only photo, set first and last 
  //pointers to this photo.
  if(firstPhoto == NULL)
  {
    firstPhoto = newPhoto;
    lastPhoto = newPhoto;
  }
  //else append to end of list
  else
  {
    lastPhoto->setNext(newPhoto);
    lastPhoto = newPhoto;
  }
}
//==============================================
void Subalbum::photoMoved(Photo* val)
{
  //decrease counter
  numPhotos--;
 
 //walk through list of photos and find specified photo
  Photo* current = firstPhoto;
  Photo* prev = NULL;
  while(current != NULL &&
        current != val)
  {
    prev = current;
    current = current->getNext();
  }

  if(current == val)
  {
    //update prev next pointer
    if(prev != NULL)
      prev->setNext(current->getNext());
    
    //update first and last pointers if necessary
    if(current == firstPhoto)
      firstPhoto = current->getNext();
    if(current == lastPhoto)
      lastPhoto = prev;
  }
}
//==============================================
void Subalbum::syncPhotoList(PhotoWidget* item)
{
  //base case, no items
  if(item == NULL)
  {
    firstPhoto = NULL;
    lastPhoto = NULL;
    return;
  }
  
  //set first and last pointers
  firstPhoto = item->getPhoto();
  firstPhoto->setNext(NULL);
  lastPhoto = firstPhoto;
    
  //set all next pointers
  while(item->nextItem() != NULL)
  {
    item->getPhoto()->setNext( ((PhotoWidget*)item->nextItem())->getPhoto() );
    item = (PhotoWidget*)item->nextItem();
    lastPhoto = item->getPhoto();
    lastPhoto->setNext(NULL);
  }
}
//==============================================
int Subalbum::getSubalbumNumber()
{
  return number;
}
//==============================================
int Subalbum::getNumPhotos()
{
  return numPhotos;;
}
//==============================================
int Subalbum::getNumLoadedPhotos()
{
  return loadedPhotos;;
}
//==============================================
void Subalbum::resetNumLoadedPhotos()
{
  loadedPhotos = numPhotos;
}
//==============================================
