//==============================================
//  copyright            : (C) 2003 by Will Stokes
//==============================================
//  This program is free software; you can redistribute it 
//  and/or modify it under the terms of the GNU General 
//  Public License as published by the Free Software 
//  Foundation; either version 2 of the License, or  
//  (at your option) any later version.         
//
//  As a special exception, Will Stokes gives permission to 
//  link this program with Qt non-commercial edition, and 
//  distribute the resulting executable, without including the 
//  source code for the Qt non-commercial edition in the 
//  source distribution. 
//==============================================

//Systemwide includes
#include <qwidget.h>
#include <qlayout.h>
#include <qlabel.h>

//Projectwide includes
#include "layoutWidget.h"
#include "window.h"
#include "subalbumsWidget.h"
#include "subalbumWidget.h"
#include "photoEditWidget.h"
#include "../backend/subalbum.h"
#include "../backend/photo.h"

#include <iostream.h>

//==============================================
LayoutWidget::LayoutWidget(QWidget *parent, 
                           const char* name ) : QWidget(parent,name)
{
  window = (Window*)parent;
  
  subalbums = new SubalbumsWidget( this, "subalbums" );
  subalbum = NULL;
  
  //place the subalbums list and subalbum view in grid
  grid = new QGridLayout( this, 1, 2, 0 );
  grid->addWidget( subalbums, 0, 0 );

  //create a photo edit widget, when need the current subalbum can be
  //removed from the grid and the photo edit widget can be inserted, then
  //exchanged again when edting concludes.
  photoEdit = new PhotoEditWidget( this, "photo edit" );
  grid->addWidget( photoEdit, 0, 1 );
  photoEdit->hide();
    
  grid->setColStretch( 1, 1 );

  grid->addColSpacing(0, 100 );

  //set the background of the widget to be white
  setPaletteBackgroundColor( QColor(255, 255, 255) );
}
//==============================================
void LayoutWidget::updateSubalbum(Subalbum* salbum, bool oldExists)
{
  //if new selection is same as old selection do nothing
  if(subalbum != NULL && salbum == subalbum->getSubalbum())
    return;
    
  //if a subalbum previously displayed update it
  if(subalbum != NULL)
  {
    //sync up old data
    if(oldExists)
      subalbum->syncPhotos();
    
    //if new subalbum exists update subalbum
    if(salbum != NULL)
    {
      subalbum->setSubalbum(salbum);
    }
    //else just destroy old subalbum view
    else
    {
      delete subalbum;
      subalbum = NULL;
    }
  }
  //else create a new subalbum widget and populate it
  else if(salbum != NULL)
  {
    //create new subalbum widget
    subalbum = new SubalbumWidget( salbum, this, "subalbum" );

    //insert into layout
    grid->addWidget( subalbum, 0, 1 );
    subalbum->show();
    subalbum->refreshPhotos();
  }
}
//==============================================
void LayoutWidget::updateSubalbumName(const QString& val)
{
  subalbums->updateSubalbumName(val);
}
//==============================================
void LayoutWidget::updateSubalbumImage( QPixmap* val)
{
  subalbums->updateSubalbumThumbnail(val);
}
//==============================================
SubalbumWidget* LayoutWidget::getSubalbum()
{
  return subalbum;
}
//==============================================
SubalbumsWidget* LayoutWidget::getSubalbums()
{
  return subalbums;
}
//==============================================
Window* LayoutWidget::getWindow()
{
  return window;
}
//==============================================
void LayoutWidget::refresh()
{
  subalbums->refresh();
}
//==============================================
void LayoutWidget::editPhoto(Photo* photo)
{
  //never edit null photos, this should never happen but it's a sanity check anyways
  if(photo == NULL)
    return;
    
  //if a subalbum exists hide it
  if(subalbum != NULL)
    subalbum->hide();
    
  //set the photo pointer for the photo edit widget
  photoEdit->setPhoto(photo);
  
  //unhide the photo edit widget
  photoEdit->show();    
}
//==============================================
void LayoutWidget::stopEdit(bool oldExists)
{
   //hide edit window, show subalbum window
  //refresh thumbnail and text for selected image
  photoEdit->hide();
  if(subalbum != NULL)
  {
    subalbum->show();
   if(oldExists)
     subalbum->refreshSelectedPhotos();
  }  
}
//==============================================
