//==============================================
//  copyright            : (C) 2003 by Will Stokes
//==============================================
//  This program is free software; you can redistribute it 
//  and/or modify it under the terms of the GNU General 
//  Public License as published by the Free Software 
//  Foundation; either version 2 of the License, or  
//  (at your option) any later version.         
//
//  As a special exception, Will Stokes gives permission to 
//  link this program with Qt non-commercial edition, and 
//  distribute the resulting executable, without including the 
//  source code for the Qt non-commercial edition in the 
//  source distribution. 
//==============================================

//Systemwide includes
#include <qpixmap.h>
#include <qlabel.h>
#include <qfont.h>
#include <qlistbox.h>
#include <qlayout.h>
#include <qapplication.h>

//Projectwide includes
#include "saveDialog.h"
#include "../config.h"

//==============================================
SaveDialog::SaveDialog( QWidget *parent, const char* name ) :
                        QWidget(parent,name)

{
  //create and set application icon
  applicationIcon = new QPixmap(QString(IMAGE_PATH)+"albumShaperIcon.png");
  setIcon( *applicationIcon );
  setCaption( "Saving...");

  //create widgts and place in window
  saving = new QLabel( this );
  saving->setText( "Saving..." );
  saving->setFont( QFont( "Time", 12, QFont::Bold ) );
  
  entries = new QListBox( this );
  entries->setSelectionMode( QListBox::NoSelection ); 
  
  //place all labels in grid layout
  grid = new QGridLayout( this, 2, 1, 0 );
  grid->addWidget( saving, 0, 0, Qt::AlignLeft);
  grid->addWidget( entries, 1, 0);
  
  //set the background of the widget to be white
  setPaletteBackgroundColor( QColor(255, 255, 255) );

  //set defaults
  lastMessageSubalbumHTML = false;
  lastMessageSubalbumPhoto = false;
  lastSubalbum = -1;
}
//==============================================
SaveDialog::~SaveDialog()
{
  delete applicationIcon;
}
//==============================================
void SaveDialog::printMessage( QString message )
{
  entries->insertItem( message );
  entries->triggerUpdate( false );
  lastMessageSubalbumHTML = false;
  lastMessageSubalbumPhoto = false;
  lastSubalbum = -1;
  repaint();
  qApp->processEvents();
}
//==============================================
void SaveDialog::printSubalbumHTML( int val )
{
  if(lastMessageSubalbumHTML)
  {
    entries->changeItem( QString("Subalbum %1 HTML").arg(val), entries->count() -1);
  }
  else
  {
    entries->insertItem( QString("Subalbum %1 HTML").arg(val) );
    lastMessageSubalbumHTML = true;
  }
  
  lastMessageSubalbumPhoto = false;
  lastSubalbum = -1;
  repaint();
  qApp->processEvents();
}
//==============================================
void SaveDialog::printSubalbumPhoto( int subalbum, int photo )
{
  if(lastMessageSubalbumPhoto && lastSubalbum == subalbum)
  {
    entries->changeItem( QString("Subalbum %1 photo %2").arg(subalbum).arg(photo), entries->count() -1 );
  }
  else
  {
    entries->insertItem( QString("Subalbum %1 photo %2").arg(subalbum).arg(photo) );
    lastMessageSubalbumPhoto = true;
    lastSubalbum = subalbum;
  }

  lastMessageSubalbumHTML = false;
  repaint();
  qApp->processEvents();
}
//==============================================

