//==============================================
//  copyright            : (C) 2003 by Will Stokes
//==============================================
//  This program is free software; you can redistribute it 
//  and/or modify it under the terms of the GNU General 
//  Public License as published by the Free Software 
//  Foundation; either version 2 of the License, or  
//  (at your option) any later version.         
//
//  As a special exception, Will Stokes gives permission to 
//  link this program with Qt non-commercial edition, and 
//  distribute the resulting executable, without including the 
//  source code for the Qt non-commercial edition in the 
//  source distribution. 
//==============================================

//Systemwide includes
#include <qpixmap.h>
#include <qstring.h>
#include <qpainter.h>
#include <qiconview.h>

//Projectwide includes
#include "subalbumPreviewWidget.h"
#include "subalbumsIconView.h"
#include "../backend/subalbum.h"
#include "../backend/photo.h"
#include "subalbumsWidget.h"
#include "layoutWidget.h"
#include "subalbumWidget.h"
#include "photoWidget.h"

#include <iostream.h>

//==============================================
SubalbumPreviewWidget::SubalbumPreviewWidget( SubalbumsIconView* parent, 
                                              Subalbum* salbum ) :
                                              QIconViewItem(parent, 
                                                            salbum->getName(),
                                                            *salbum->getRepresentativeImage(MEDIUM))
{
  this->parent = parent;
  subalbum = salbum;
}
//==============================================
Subalbum* SubalbumPreviewWidget::getSubalbum()
{
  return subalbum;
}
//==============================================
void SubalbumPreviewWidget::paintItem( QPainter* p, 
                                       const QColorGroup& cg)                          
{
  if(isSelected())
  {
    //draw red border around item
    int align = AlignHCenter;
    align |= WordBreak | BreakAnywhere;
    p->drawPixmap(pixmapRect(false),*pixmap());
    p->drawText(textRect(false),align, text());
    p->setPen( red );
    p->drawRect(rect());
  }
  else
  {
    QIconViewItem::paintItem ( p, cg);
  }
}
//==============================================
void SubalbumPreviewWidget::dropped( QDropEvent *e, const QValueList<QIconDragItem> & ) 
{
  if(e->source()->parentWidget() == parent)
  {
    if(e->pos().y() < (y() + (height()/2)))
    {
      parent->currentItem()->move(x(), y() - 1);
    }
    else
    {
      parent->currentItem()->move(x(), y() + (height()/2) + 1);
    }
  }
  else
  {
    //if the source of the items is the current subalbum icon view and
    //this is a differnt subalbum then
    //move photos from that subalbum to this one
    if(!isSelected() && e->source()->parentWidget() == ((LayoutWidget*)(parent->parentWidget()->parentWidget()))->getSubalbum()->getPhotos())
    {
      //iterate over all selected photos, inserting each 
      //into this subalbum, removing from old subalbum,
      //and deleting old photo widgets
      SubalbumWidget* oldSubalbumWidget = ((LayoutWidget*)(parent->parentWidget()->parentWidget()))->getSubalbum();
      Subalbum* oldSubalbum = oldSubalbumWidget->getSubalbum();
      QIconViewItem* current = oldSubalbumWidget->getPhotos()->firstItem();
    
      while(current != NULL)
      {
       //found a selected photo
        if(current->isSelected())      
        {
          //get pointer to photo
          Photo* photo = ((PhotoWidget*)current)->getPhoto();
                
          //remove photo from that subalbum
          oldSubalbum->photoMoved(photo);
                
          //add photo to this subalbum
          subalbum->addPhoto(photo);
        
          //delete photo widget and rearrange photos
          QIconViewItem* temp = current;
          current = current->nextItem();
          delete temp;
        
          oldSubalbumWidget->getPhotos()->arrangeItemsInGrid();
        }
        else
          current = current->nextItem();
      }
    }
  }
}
//==============================================
bool SubalbumPreviewWidget::acceptDrop( const QMimeSource *) const
{
  return true;
}
//==============================================
int SubalbumPreviewWidget::compare ( QIconViewItem * i ) const
{
  if(pos().y() >= i->pos().y())
  {  return 1; }
  else
  { return -1; }
}
//==============================================
