//==============================================
//  copyright            : (C) 2003 by Will Stokes
//==============================================
//  This program is free software; you can redistribute it 
//  and/or modify it under the terms of the GNU General 
//  Public License as published by the Free Software 
//  Foundation; either version 2 of the License, or  
//  (at your option) any later version.         
//
//  As a special exception, Will Stokes gives permission to 
//  link this program with Qt non-commercial edition, and 
//  distribute the resulting executable, without including the 
//  source code for the Qt non-commercial edition in the 
//  source distribution. 
//==============================================

//Systemwide includes
#include <qlayout.h>
#include <qlabel.h>
#include <qfont.h>
#include <qframe.h>
#include <qpushbutton.h>
#include <qpixmap.h>
#include <qlineedit.h>
#include <qfiledialog.h>
#include <qcursor.h>
#include <qapplication.h>

//Projectwide includes
#include "../backend/subalbum.h"
#include "../backend/photo.h"
#include "window.h"
#include "titleWidget.h"
#include "layoutWidget.h"
#include "subalbumWidget.h"
#include "subalbumsWidget.h"
#include "photoWidget.h"
#include "questionDialog.h"
#include "photosIconView.h"
#include "../config.h"

#include <iostream.h>
//==============================================
SubalbumWidget::SubalbumWidget(Subalbum* salbum,
                               QWidget *parent, 
                               const char* name ) : 
                               QWidget(parent,name)
{
  //store subalbum pointer
  subalbum = salbum;

  //store layout pointer
  layout = (LayoutWidget*)parent;

  //create subalbum name and description label and text entries
  subalbumName = new QLabel( this );
  subalbumName->setText( "Subalbum Name:" );
  subalbumName->setFont( QFont( "Times", 12, QFont::Bold ) );
  subalbumNameVal = new QLineEdit( this );
  subalbumNameVal->setFont( QFont( "Times", 12, QFont::Bold ) );
  subalbumNameVal->setText(  subalbum->getName() );
  connect( subalbumNameVal, SIGNAL(textChanged(const QString&)),
           SLOT(updateName(const QString&)) );

  subalbumDescription = new QLabel( this );
  subalbumDescription->setText( "Description:" );
  subalbumDescription->setFont( QFont( "Times", 12, QFont::Bold ) );
  subalbumDescriptionVal = new QLineEdit( this );
  subalbumDescriptionVal->setFont( QFont( "Times", 12, QFont::Bold ) );
  subalbumDescriptionVal->setText(  subalbum->getDescription() );
  connect( subalbumDescriptionVal, SIGNAL(textChanged(const QString&)),
           SLOT(updateDescription(const QString&)) );

  //create set image button
  representativeImageText = new QLabel( this );
  representativeImageText->setText( "Thumbnail:" );
  representativeImageText->setFont( QFont( "Times", 12, QFont::Bold ) );
  setImageImage = new QPixmap( QString(IMAGE_PATH)+"set.png" );
  setImage = new QPushButton( this );
  setImage->setPixmap( *setImageImage );
  setImage->setSizePolicy( QSizePolicy::Maximum, QSizePolicy::Maximum );      
  setImage->setEnabled(false);
  connect( setImage, SIGNAL(clicked()), SLOT(setImageAction()) );
  
  //create representative image
  representativeImage = subalbum->getRepresentativeImage(SMALL);
  representativeLogo = new QLabel( this );
  representativeLogo->setPixmap( *representativeImage );

  //create photo collection
  photos = new PhotosIconView( this );
  
  //establish a top-down view such that the scrollbar is always placed on the right
  photos->setArrangement( QIconView::LeftToRight );
  photos->setVScrollBarMode( QScrollView::AlwaysOn );
  
  //allow multiple photos to be selected with control and shift keys
  photos->setSelectionMode( QIconView::Extended ) ;
  
  //set auto-scroll on for drag-n-drop
  photos->setDragAutoScroll(true);
  photos->setAcceptDrops(true);

  //whenever photo names are changed sync with backend object
  connect( photos, SIGNAL(itemRenamed( QIconViewItem *)), 
           this, SLOT(rearrangeAndSaveCurrent( QIconViewItem *)));
           
  //connect selectionChanged signal to update buttons method
  connect( photos, SIGNAL(selectionChanged()),
           SLOT(updateButtons()) );

  //connect rightButtonClicked signal to update buttons method
  connect( photos, SIGNAL(rightButtonClicked(QIconViewItem*, const QPoint&)),
           SLOT(updateButtons()) );
           
  //if the set button is clicked set the representative image for the subalbum
  connect( setImage, SIGNAL(clicked()), SLOT(setImageAction()) );

  //connect drop event on iconview to reorder slot
  connect( photos, SIGNAL(itemHasMoved()), SLOT(reorder()) );  
      
  buttons = new QFrame(this);
  
  addImageImage = new QPixmap( QString(IMAGE_PATH)+"add.png" );
  addImage = new QPushButton( buttons );
  addImage->setPixmap( *addImageImage );
  addImage->setSizePolicy( QSizePolicy::Maximum, QSizePolicy::Maximum );    
  connect( addImage, SIGNAL(clicked()), SLOT(addImageAction()) );
  
  removeImageImage = new QPixmap( QString(IMAGE_PATH)+"remove.png" );
  removeImage = new QPushButton( buttons );
  removeImage->setPixmap( *removeImageImage );
  removeImage->setSizePolicy( QSizePolicy::Maximum, QSizePolicy::Maximum );    
  connect( removeImage, SIGNAL(clicked()), SLOT(removeImageAction()) );
  
  rotate90ImageImage = new QPixmap( QString(IMAGE_PATH)+"rotate90.png" );
  rotate90Image = new QPushButton( buttons );
  rotate90Image->setPixmap( *rotate90ImageImage );
  rotate90Image->setSizePolicy( QSizePolicy::Maximum, QSizePolicy::Maximum );    
  connect( rotate90Image, SIGNAL(clicked()), SLOT(rotate90ImageAction()) );
  
  rotate270ImageImage = new QPixmap( QString(IMAGE_PATH)+"rotate270.png" );
  rotate270Image = new QPushButton( buttons );
  rotate270Image->setPixmap( *rotate270ImageImage );
  rotate270Image->setSizePolicy( QSizePolicy::Maximum, QSizePolicy::Maximum );    
  connect( rotate270Image, SIGNAL(clicked()), SLOT(rotate270ImageAction()) );
  
  flipHorizontallyImageImage = new QPixmap( QString(IMAGE_PATH)+"flipHorizontally.png" );
  flipHorizontallyImage = new QPushButton( buttons );
  flipHorizontallyImage->setPixmap( *flipHorizontallyImageImage );
  flipHorizontallyImage->setSizePolicy( QSizePolicy::Maximum, QSizePolicy::Maximum );    
  connect( flipHorizontallyImage, SIGNAL(clicked()), SLOT(flipHorizontallyImageAction()) );
  
  flipVerticallyImageImage = new QPixmap( QString(IMAGE_PATH)+"flipVertically.png" );
  flipVerticallyImage = new QPushButton( buttons );
  flipVerticallyImage->setPixmap( *flipVerticallyImageImage );
  flipVerticallyImage->setSizePolicy( QSizePolicy::Maximum, QSizePolicy::Maximum );    
  connect( flipVerticallyImage, SIGNAL(clicked()), SLOT(flipVerticallyImageAction()) );
  
  editImage = new QPixmap( QString(IMAGE_PATH)+"editPhoto.png" );
  editButton = new QPushButton( buttons );
  editButton->setPixmap( *editImage );
  editButton->setSizePolicy( QSizePolicy::Maximum, QSizePolicy::Maximum );    
  connect( editButton, SIGNAL(clicked()), SLOT(editAction()) );
  
  //place all items in grid layout
  grid = new QGridLayout( this, 5, 5, 0 );
  grid->addWidget( subalbumName, 0, 0, Qt::AlignLeft );
  grid->addWidget( subalbumNameVal, 0, 1 );
  grid->addWidget( subalbumDescription, 1, 0, Qt::AlignLeft );
  grid->addWidget( subalbumDescriptionVal, 1, 1 );
  grid->addWidget( representativeImageText, 0, 2, Qt::AlignLeft );
  grid->addMultiCellWidget( setImage, 1, 1, 2, 2 );
  grid->addMultiCellWidget( representativeLogo, 0, 2, 3, 3 );
  grid->addMultiCellWidget( photos, 3, 3, 0, 4 );

  grid2 = new QGridLayout( buttons, 1, 9, 0 );
  grid2->addWidget( addImage, 0, 1, Qt::AlignLeft );
  grid2->addWidget( removeImage, 0, 2, Qt::AlignLeft );
  grid2->addWidget( rotate90Image, 0, 3, Qt::AlignLeft );
  grid2->addWidget( rotate270Image, 0, 4, Qt::AlignLeft );
  grid2->addWidget( flipHorizontallyImage, 0, 5, Qt::AlignLeft );
  grid2->addWidget( flipVerticallyImage, 0, 6, Qt::AlignLeft );
  grid2->addWidget( editButton, 0, 7, Qt::AlignLeft );
  
  //set first and last columns of grid2 to take up remaining space
  grid2->setColStretch( 0, 1 );
  grid2->setColStretch( 8, 1 );
  grid->addMultiCellWidget( buttons, 4, 4, 0, 4 );
  
  //set the second column to stretch to fill any blank space
  grid->setColStretch( 4, 1 );

  //set the last row to stretch to fill any blank space
  grid->setRowStretch( 3, 1 );
   
  //Set the second column, the actual subalbum name and description
  //to have a minimum width
  grid->addColSpacing(1, 300 );
  
  //set the background of the widget to be white
  setPaletteBackgroundColor( QColor(255, 255, 255) );
  
  //by default no selected images so disable all buttons besides add
  removeImage->setEnabled(false);  
  rotate90Image->setEnabled(false);  
  rotate270Image->setEnabled(false);  
  flipHorizontallyImage->setEnabled(false);  
  flipVerticallyImage->setEnabled(false);  
  editButton->setEnabled(false);  
}
//==============================================
SubalbumWidget::~SubalbumWidget()
{
  delete setImageImage;
  delete addImageImage;
  delete removeImageImage;
  delete rotate90ImageImage;
  delete rotate270ImageImage;
  delete flipHorizontallyImageImage;
  delete flipVerticallyImageImage;
}
//==============================================
void SubalbumWidget::setSubalbum(Subalbum* salbum)
{
  //set new subalbum pointer
  subalbum = salbum;
  
  //update representative image, subalbum name and description
  subalbumNameVal->setText(  subalbum->getName() );
  subalbumDescriptionVal->setText(  subalbum->getDescription() );
  representativeImage = subalbum->getRepresentativeImage(SMALL);
  representativeLogo->setPixmap( *representativeImage );
  
  //update photo listing
  refreshPhotos();
  
  //disable/enable buttons as necessary
  updateButtons();
}
//==============================================
//called when user clicks "set image" button
void SubalbumWidget::setImageAction()
{
  //---------------------------------------------------------    
  //get handle on photo widget
  Photo* selectedPhoto = getSelectedPhoto();
  if(selectedPhoto == NULL)
    return;
  //---------------------------------------------------------    
  //set representative images
  subalbum->setRepresentativeImages( selectedPhoto->getImage(THUMBNAIL) );
  //---------------------------------------------------------    
  //update onscreen images
  representativeLogo->setPixmap( *(subalbum->getRepresentativeImage(SMALL)) );
  layout->updateSubalbumImage( subalbum->getRepresentativeImage(MEDIUM) );
  //---------------------------------------------------------    
}
//==============================================
void SubalbumWidget::updateName( const QString& val )
{
  subalbum->setName(val);
  layout->updateSubalbumName(val);
}
//==============================================
void SubalbumWidget::updateDescription( const QString& val )
{
  subalbum->setDescription(val);
}
//==============================================
void SubalbumWidget::addImageAction()
{
  //set busy flag and deactivate menu's/buttons, and selecting photos
  layout->getWindow()->getTitle()->setBusy(true);
  layout->getSubalbums()->updateButtons(false);
  updateButtons(false);
  photos->setSelectionMode( QIconView::NoSelection ) ;
    
  //Get filename from user
  QStringList fileNames = QFileDialog::getOpenFileNames("Images (*.gif *.jpg *.jpeg *.png *.xpm *.GIF *.JPG *.JPEG *.PNG *.XPM)",
                                                        NULL,
                                                        this,
                                                        "add photos dialog",
                                                        "Select photos");
      
  qApp->setOverrideCursor( QCursor(Qt::WaitCursor));
  
  //iterate through each file and add to album
  QStringList::iterator it;
  for(it = fileNames.begin(); it != fileNames.end(); it++ )
  {
    //if successful adding photo add widget
    if(subalbum->addPhoto(*it))
    {  
      PhotoWidget* p =  new PhotoWidget( photos, subalbum->getLast() );
      p->setRenameEnabled(true);
      photos->ensureItemVisible(p);
      qApp->processEvents();
    }
  }
  photos->arrangeItemsInGrid();
  
  //unset busy flag and activate menu's/buttons
  layout->getWindow()->getTitle()->setBusy(false);
  layout->getSubalbums()->updateButtons(true);
  updateButtons(true);
  photos->setSelectionMode( QIconView::Extended ) ;
  qApp->restoreOverrideCursor();
}
//==============================================
void SubalbumWidget::removeImageAction()
{
  //set busy flag and deactivate menu's/buttons
  layout->getWindow()->getTitle()->setBusy(true);
  layout->getSubalbums()->updateButtons(false);
  updateButtons(false);
  photos->setSelectionMode( QIconView::NoSelection ) ;
  
  //check if user is sure
  QuestionDialog sure( "Remove selected photos?",
                       "Once removed photos cannot be restored. Furthermore apon resaving they are physically removed from your album.",
                       "warning.png",
                       this );    
  //if user say yes then delete subalbum and refresh
  if(sure.exec())  
  {
    qApp->setOverrideCursor( QCursor(Qt::WaitCursor));
    //iterate through all photos and remove those that are selected
    QIconViewItem* current = photos->firstItem();
    QIconViewItem* temp;
    while(current != NULL)
    {
      //if not selected move on
      if(!current->isSelected())
      {
        current = current->nextItem();
        continue;
      }
      
      //get next pointer  
      temp = current->nextItem();
    
      //grab point to backend photo
      Photo* phto = ((PhotoWidget*)current)->getPhoto();
    
      //delete photo widget
      delete current;
      current = temp;
    
      //delete backend photo
      subalbum->removePhoto(phto); 
      
      //check if any photos selected, if not deactivate all buttons besides add button
      updateButtons();
    }
  
    photos->arrangeItemsInGrid();
  
    //unset busy flag and activate menu's/buttons
    qApp->restoreOverrideCursor();
  }
    
  layout->getWindow()->getTitle()->setBusy(false);
  layout->getSubalbums()->updateButtons(true);
  updateButtons(true);
  photos->setSelectionMode( QIconView::Extended ) ;
}
//==============================================
void SubalbumWidget::rotate90ImageAction()
{
  //set busy flag and deactivate menu's/buttons
  qApp->setOverrideCursor( QCursor(Qt::WaitCursor));
  layout->getWindow()->getTitle()->setBusy(true);
  layout->getSubalbums()->updateButtons(false);
  photos->setSelectionMode( QIconView::NoSelection ) ;
  updateButtons(false);
  
  //rotate the selected photos
  QIconViewItem* current = photos->firstItem();
  while(current != NULL)
  {
    if(current->isSelected())
    {
      ((PhotoWidget*)current)->getPhoto()->rotate90();
      photos->ensureItemVisible(((PhotoWidget*)current));
      ((PhotoWidget*)current)->updateImage();
      qApp->processEvents();
    }
    
    //move to next item
    current = current->nextItem();
  }
  
  //not busy any more
  layout->getWindow()->getTitle()->setBusy(false);
  layout->getSubalbums()->updateButtons(true);
  updateButtons(true);
  photos->setSelectionMode( QIconView::Extended ) ;
  qApp->restoreOverrideCursor();
}
//==============================================
void SubalbumWidget::rotate270ImageAction()
{
  //set busy flag and deactivate menu's/buttons
  qApp->setOverrideCursor( QCursor(Qt::WaitCursor));
  layout->getWindow()->getTitle()->setBusy(true);
  layout->getSubalbums()->updateButtons(false);
  photos->setSelectionMode( QIconView::NoSelection ) ;
  updateButtons(false);
  
  //rotate the selected photos
  QIconViewItem* current = photos->firstItem();
  while(current != NULL)
  {
    if(current->isSelected())
    {
      ((PhotoWidget*)current)->getPhoto()->rotate270();
      photos->ensureItemVisible(((PhotoWidget*)current));
      ((PhotoWidget*)current)->updateImage();
      qApp->processEvents();
    }
    
    //move to next item
    current = current->nextItem();
  }
  
  //not busy any more
  layout->getWindow()->getTitle()->setBusy(false);
  layout->getSubalbums()->updateButtons(true);
  updateButtons(true);
  photos->setSelectionMode( QIconView::Extended ) ;
  qApp->restoreOverrideCursor();
}
//==============================================
void SubalbumWidget::flipHorizontallyImageAction()
{
  //set busy flag and deactivate menu's/buttons
  qApp->setOverrideCursor( QCursor(Qt::WaitCursor));
  layout->getWindow()->getTitle()->setBusy(true);
  layout->getSubalbums()->updateButtons(false);
  photos->setSelectionMode( QIconView::NoSelection ) ;
  updateButtons(false);
  
  //rotate the selected photos
  QIconViewItem* current = photos->firstItem();
  while(current != NULL)
  {
    if(current->isSelected())
    {
      ((PhotoWidget*)current)->getPhoto()->flipHorizontally();
      photos->ensureItemVisible(((PhotoWidget*)current));
      ((PhotoWidget*)current)->updateImage();
      qApp->processEvents();
    }
    
    //move to next item
    current = current->nextItem();
  }
  
  //not busy any more
  layout->getWindow()->getTitle()->setBusy(false);
  layout->getSubalbums()->updateButtons(true);
  updateButtons(true);
  photos->setSelectionMode( QIconView::Extended ) ;
  qApp->restoreOverrideCursor();
}
//==============================================
void SubalbumWidget::flipVerticallyImageAction()
{
  //set busy flag and deactivate menu's/buttons
  qApp->setOverrideCursor( QCursor(Qt::WaitCursor));
  layout->getWindow()->getTitle()->setBusy(true);
  layout->getSubalbums()->updateButtons(false);
  photos->setSelectionMode( QIconView::NoSelection ) ;
  updateButtons(false);
  
  //rotate the selected photos
  QIconViewItem* current = photos->firstItem();
  while(current != NULL)
  {
    if(current->isSelected())
    {
      ((PhotoWidget*)current)->getPhoto()->flipVertically();
      photos->ensureItemVisible(((PhotoWidget*)current));
      ((PhotoWidget*)current)->updateImage();
      qApp->processEvents();
    }
    
    //move to next item
    current = current->nextItem();
  }
  
  //not busy any more
  layout->getWindow()->getTitle()->setBusy(false);
  layout->getSubalbums()->updateButtons(true);
  updateButtons(true);
  photos->setSelectionMode( QIconView::Extended ) ;
  qApp->restoreOverrideCursor();
}
//==============================================
void SubalbumWidget::editAction()
{
  Photo* selected = getSelectedPhoto();
  if(selected != NULL)
    layout->editPhoto( selected );
  else
  {
    //bring up dialog scolding user for trying to edit either no or multiple photos at once
  }
}
//==============================================
void SubalbumWidget::refreshPhotos()
{
  //remove all thumbnails
  photos->clear();

  //insert photo thumbnails
  Photo* currentPhoto = subalbum->getFirst();
  while(currentPhoto != NULL)
  {
    PhotoWidget* p = new PhotoWidget( photos, currentPhoto );
    p->setRenameEnabled(true);
    currentPhoto = currentPhoto->getNext();
  }
  
  photos->arrangeItemsInGrid();
}
//==============================================
void SubalbumWidget::refreshSelectedPhotos()
{
  QIconViewItem* current = photos->firstItem();
  while(current != NULL)
  {
    //found a selected item!
    if(current->isSelected())
    {
      ((PhotoWidget*)current)->updateImage();
      ((PhotoWidget*)current)->updateDescription();
    }
    
    //move to next item
    current = current->nextItem();
  }
}
//==============================================
void SubalbumWidget::syncPhotos()
{
  PhotoWidget* current = (PhotoWidget*)photos->firstItem(); 
  while(current != NULL)
  {
    current->sync();
    current = (PhotoWidget*)current->nextItem();  
  }
}
//==============================================
Subalbum* SubalbumWidget::getSubalbum()
{
  return subalbum;
}
//==============================================
Photo* SubalbumWidget::getSelectedPhoto()
{
  //determine if one photo is selected
  int numSelected = 0;
  QIconViewItem* current = photos->firstItem();
  QIconViewItem* selected = NULL;
  while(current != NULL)
  {
    //found a selected item!
    if(current->isSelected())
    {
      numSelected++;
      selected = current;
    }
    
    //if more than one found then bail!
    if(numSelected > 1)
      break;
    
    //move to next item
    current = current->nextItem();
  }
  
  //if one item is selected then return photo pointer
  if(numSelected == 1)
  {
    return ((PhotoWidget*)selected)->getPhoto();
  }
  else
  {
    return NULL;
  }
}
//==============================================
void SubalbumWidget::resizeEvent( QResizeEvent * )
{
  photos->arrangeItemsInGrid();  
}
//==============================================
QIconView* SubalbumWidget::getPhotos()
{
  return photos;
}
//==============================================
void SubalbumWidget::rearrangeAndSaveCurrent(QIconViewItem* item)
{
  ((PhotoWidget*)item)->sync();
  photos->arrangeItemsInGrid();  
}
//==============================================
void SubalbumWidget::reorder()
{
  //so item has been moved, reorder linked list of items as necessary
  photos->sort( true );
  photos->arrangeItemsInGrid();
  
  //sync lists
  subalbum->syncPhotoList((PhotoWidget*)photos->firstItem());
}
//==============================================
void SubalbumWidget::updateButtons()
{
  bool anySelected = false;
  QIconViewItem* current = photos->firstItem();
  while(current != NULL)
  {
    if(current->isSelected())
    {
     anySelected = true;
     break;
    }
    
    //move to next item
    current = current->nextItem();
  }

  if(!anySelected)
  {
    removeImage->setEnabled(false);  
    rotate90Image->setEnabled(false);  
    rotate270Image->setEnabled(false);  
    flipHorizontallyImage->setEnabled(false);  
    flipVerticallyImage->setEnabled(false);  
    editButton->setEnabled(false);  
    layout->getWindow()->getTitle()->setSetButtonState(false);
    setImage->setEnabled(false);
  }
  else
  {
    removeImage->setEnabled(true);  
    rotate90Image->setEnabled(true);  
    rotate270Image->setEnabled(true);  
    flipHorizontallyImage->setEnabled(true);  
    flipVerticallyImage->setEnabled(true);  
    editButton->setEnabled(true);  
    layout->getWindow()->getTitle()->setSetButtonState(true);
    setImage->setEnabled(true);
  }
}
//==============================================
void SubalbumWidget::updateButtons(bool enable)
{
  if(!enable)
  {
    buttonsState = rotate90Image->isEnabled();
    addImage->setEnabled(enable);
    removeImage->setEnabled(enable);  
    rotate90Image->setEnabled(enable);  
    rotate270Image->setEnabled(enable);  
    flipHorizontallyImage->setEnabled(enable);  
    flipVerticallyImage->setEnabled(enable);  
    editButton->setEnabled(enable);  
    setImage->setEnabled(enable);
  }
  else
  {
    addImage->setEnabled(enable);
    removeImage->setEnabled(buttonsState);  
    rotate90Image->setEnabled(buttonsState);  
    rotate270Image->setEnabled(buttonsState);  
    flipHorizontallyImage->setEnabled(buttonsState);  
    flipVerticallyImage->setEnabled(buttonsState);  
    editButton->setEnabled(buttonsState);  
    setImage->setEnabled(buttonsState);
  }
}
//==============================================
