//==============================================
//  copyright            : (C) 2003 by Will Stokes
//==============================================
//  This program is free software; you can redistribute it 
//  and/or modify it under the terms of the GNU General 
//  Public License as published by the Free Software 
//  Foundation; either version 2 of the License, or  
//  (at your option) any later version.         
//
//  As a special exception, Will Stokes gives permission to 
//  link this program with Qt non-commercial edition, and 
//  distribute the resulting executable, without including the 
//  source code for the Qt non-commercial edition in the 
//  source distribution. 
//==============================================

#ifndef GUI_SUBALBUMSWIDGET_H
#define GUI_SUBALBUMSWIDGET_H

//forward declarations
class QGridLayout;
class QLabel;
class QIconViewItem;
class QPixmap;
class QPushButton;
class LayoutWidget;
class SubalbumsIconView;

#include <qwidget.h>

//=====================================
/*! \brief Columnview of all subalbums in album.
 */
//=====================================

class SubalbumsWidget : public QWidget
{
Q_OBJECT

public:
  ///Creates layout
  SubalbumsWidget(QWidget *parent=0, 
                  const char* name=0);
                  
  ///Deletes objects
  ~SubalbumsWidget();

  ///Updates subalbum name
  void updateSubalbumName(const QString& val);
  
  ///Updates the current subalbums thumbnail
  void updateSubalbumThumbnail( QPixmap* val);    
  
  ///Refreshes layout
  void refresh();
  
  ///Returns parent
  LayoutWidget* getParent();
  
  ///Activates/Deactives create/delete buttons
  void updateButtons(bool enable);
//----------------------
private slots:
  ///Creates new subalbum and adds to layout
  void createAction();

  ///Deletes selected subalbum if user selects "yes"
  void deleteAction();

  ///Updates subalbum layout with user selects a differnt subalbum
  void updateSubalbumLayout( QIconViewItem* selection);
  void updateSubalbumLayout( QIconViewItem* selection, bool oldExists);

  //Rearranges subalbums in grid
  void reorder();
//----------------------
private:

  ///Grid widgets place in
  QGridLayout* grid;

  ///"Subalbums:" label
  QLabel* subalbumsText;

  ///list of subalbums
  SubalbumsIconView* subalbums;

  ///"Create" image
  QPixmap* createSubalbumImage;
  
  ///"Create" button
  QPushButton* createSubalbum;

  ///"Delete" image
  QPixmap* deleteSubalbumImage;

  ///"Delete" button  
  QPushButton* deleteSubalbum;

  ///Pointer to layoutwidget this widget is in
  LayoutWidget* layout;
  
  ///cached enabled/disabled state of buttons
  bool buttonsState;
  
  ///cached selected subalbum
  QIconViewItem* cachedSelection;
//----------------------
};
//======================

#endif //GUI_SUBALBUMSWIDGET_H
