//==============================================
//  copyright            : (C) 2003 by Will Stokes
//==============================================
//  This program is free software; you can redistribute it 
//  and/or modify it under the terms of the GNU General 
//  Public License as published by the Free Software 
//  Foundation; either version 2 of the License, or  
//  (at your option) any later version.         
//
//  As a special exception, Will Stokes gives permission to 
//  link this program with Qt non-commercial edition, and 
//  distribute the resulting executable, without including the 
//  source code for the Qt non-commercial edition in the 
//  source distribution. 
//==============================================

//Systemwide includes
#include <qwidget.h>
#include <qmenubar.h>
#include <qpopupmenu.h>
#include <qlayout.h>
#include <qlabel.h>
#include <qfont.h>
#include <qpixmap.h>
#include <qimage.h>
#include <qpushbutton.h>
#include <qlineedit.h>
#include <qapplication.h>
#include <qfiledialog.h>
#include <qdir.h>
#include <qcursor.h>

//Projectwide includes
#include "window.h"
#include "about.h"
#include "titleWidget.h"
#include "layoutWidget.h"
#include "subalbumsWidget.h"
#include "subalbumWidget.h"
#include "loadDialog.h"
#include "saveDialog.h"
#include "questionDialog.h"
#include "../backend/album.h"
#include "../backend/photo.h"
#include "../config.h"

#include <iostream.h>

//==============================================
TitleWidget::TitleWidget(QWidget *parent, 
                         const char* name ) : QFrame(parent,name)
{
  ///Store parent pointer
  window = (Window*)parent;

  ///Set about pointer to null
  about = NULL;
  
  //create file and help menus
  menu = new QMenuBar( this, "menuBar" );
  file = new QPopupMenu( this, "fileMenu" );
  NEW_ALBUM = file->insertItem( "&New", this, SLOT(newAlbum()), CTRL+Key_N );
  OPEN_ALBUM = file->insertItem( "&Open", this, SLOT(loadAlbum()), CTRL+Key_O );
  SAVE_ALBUM = file->insertItem( "&Save", this, SLOT(saveAlbum()), CTRL+Key_S );
  SAVEAS_ALBUM = file->insertItem( "&Save As", this, SLOT(saveAsAlbum()), CTRL+SHIFT+Key_S );
  file->insertItem( "&Quit", qApp, SLOT(quit()), CTRL+Key_Q);
  help = new QPopupMenu( this, "helpMenu" );
  help->insertItem( "&About", this, SLOT(aboutProgram()) );
  menu->insertItem( "&File", file );
  menu->insertSeparator();
  menu->insertItem( "&Help", help );
  
  //create backend album object
  albm = new Album();

  //create labels and text entries
  albumName = new QLabel( this );
  albumName->setText( "Album Name:" );
  albumName->setFont( QFont( "Times", 12, QFont::Bold ) );
  albumNameVal = new QLineEdit( this );
  albumNameVal->setFont( QFont( "Times", 12, QFont::Bold ) );
  connect( albumNameVal, SIGNAL(textChanged( const QString&)),
           SLOT( updateName(const QString&)) );
  
  albumDescription = new QLabel( this );
  albumDescription->setText( "Description:" );
  albumDescription->setFont( QFont( "Times", 12, QFont::Bold ) );
  albumDescriptionVal = new QLineEdit( this );
  albumDescriptionVal->setFont( QFont( "Times", 12, QFont::Bold ) );
  connect( albumDescriptionVal, SIGNAL(textChanged( const QString&)),
           SLOT( updateDescription(const QString&)) );

  albumAuthor = new QLabel( this );
  albumAuthor->setText( "Author:" );
  albumAuthor->setFont( QFont( "Times", 12, QFont::Bold ) );
  albumAuthorVal = new QLineEdit( this );
  albumAuthorVal->setFont( QFont( "Times", 12, QFont::Bold ) );
  connect( albumAuthorVal, SIGNAL(textChanged( const QString&)),
           SLOT( updateAuthor(const QString&)) );
  
  //create set image button
  representativeImageText = new QLabel( this );
  representativeImageText->setText( "Thumbnail:" );
  representativeImageText->setFont( QFont( "Times", 12, QFont::Bold ) );
  setImageImage = new QPixmap( QString(IMAGE_PATH)+"set.png" );
  setImage = new QPushButton( this );
  setImage->setPixmap( *setImageImage );
  setImage->setSizePolicy( QSizePolicy::Maximum, QSizePolicy::Maximum );      
  setImage->setEnabled(false);
  connect( setImage, SIGNAL(clicked()), SLOT(setImageAction()) );
  
  //create representative image
  representativeImage = albm->getRepresentativeImage(SMALL);
  representativeLogo = new QLabel( this );
  representativeLogo->setPixmap( *representativeImage );
           
  //create album Shaper image
  albumShaperImage = new QPixmap( QString(IMAGE_PATH)+"albumShaper.png" );
  albumShaperLogo = new QLabel( this );
  albumShaperLogo->setPixmap( *albumShaperImage );

  //Create horizontal line
  line = new QFrame(this);
  line->setFrameShadow( QFrame::Plain );
  line->setLineWidth( 2 );
  line->setFrameShape( QFrame::HLine );

  //place all labels in grid layout
  grid = new QGridLayout( this, 5, 6, 0 );
  grid->addMultiCellWidget( menu, 0, 0, 0, 5 );
  grid->addWidget( albumName,                   1, 0, Qt::AlignLeft);
  grid->addWidget( albumNameVal,                1, 1);
  grid->addWidget( albumDescription,            2, 0, Qt::AlignLeft);
  grid->addWidget( albumDescriptionVal,         2, 1);
  grid->addWidget( albumAuthor,                 3, 0, Qt::AlignLeft);
  grid->addWidget( albumAuthorVal,              3, 1);
  grid->addWidget( representativeImageText,     1, 2, Qt::AlignLeft );
  grid->addWidget( setImage,                    2, 2, Qt::AlignLeft );
  grid->addMultiCellWidget( representativeLogo, 1, 3, 3, 3 );
  grid->addMultiCellWidget( line,               4, 4, 0, 5);
  grid->addMultiCellWidget( albumShaperLogo,     1, 3, 5, 5 );

  //Set the second column, the actual album name and description and author
  //to have a minimum width
  grid->addColSpacing(1, 300 );

  //set the blank colum to stretch more than others to take
  //up leftover room
  grid->setColStretch( 4, 1 );

  //set the background of the widget to be white
  setPaletteBackgroundColor( QColor(255, 255, 255) );
  
  ///program not busy at first
  busy = false;
}
//==============================================
TitleWidget::~TitleWidget()
{
  delete setImageImage;
  delete representativeImage;
  delete albumShaperImage;
}
//==============================================
void TitleWidget::updateName( const QString& val )
{
  albm->setName(val);
}
//==============================================
void TitleWidget::updateDescription( const QString& val )
{
  albm->setDescription(val);
}
//==============================================
void TitleWidget::updateAuthor( const QString& val )
{
  albm->setAuthor(val);
}
//==============================================
void TitleWidget::setAlbum( Album* val)
{
  //delete old album
  delete albm;
  albm = val;
}
//==============================================
Album* TitleWidget::getAlbum()
{
  return albm;
}
//==============================================
//called when user clicks "set image" button
void TitleWidget::setImageAction()
{
  //---------------------------------------------------------    
  //determine if a subalbum is even selected
  SubalbumWidget* sw = window->getLayout()->getSubalbum();
  if(sw == NULL)
    return;
  //---------------------------------------------------------        
  //determine if a photo is selected
  Photo* selectedPhoto = sw->getSelectedPhoto();
  if(selectedPhoto == NULL)
    return;
  //---------------------------------------------------------     
  //set representative iamges
  QImage* img = selectedPhoto->getImage(IMAGE);
  albm->setRepresentativeImages( img );
  delete img;
  img = NULL;
  //---------------------------------------------------------    
  //update onscreen images
  representativeLogo->setPixmap( *(albm->getRepresentativeImage(SMALL)) );
  //---------------------------------------------------------    
}
//==============================================
void TitleWidget::refresh()
{
  albumNameVal->setText( albm->getName() );
  albumDescriptionVal->setText( albm->getDescription() );
  albumAuthorVal->setText( albm->getAuthor() );
  representativeLogo->setPixmap( *(albm->getRepresentativeImage(SMALL)) );
}                 
//==============================================
void TitleWidget::newAlbum()
{
  //ask if the user is sure, then proceed
  QuestionDialog sure( "New album?",
                       "Warning, this operation cannot be undone and all current work will be lost.",
                       "warning.png",
                       this );
  //if user say yes then delete subalbum and refresh
  if(sure.exec())  
  {
    //delete old album
    delete albm;
    
    //create new one
    albm = new Album();
  
    //refresh screen
    refresh();
    window->refresh();
  }
}
//==============================================
void TitleWidget::loadAlbum()
{
  //set busy flag and deactivate buttons
  setBusy(true);
  window->getLayout()->getSubalbums()->updateButtons(false);
  if(window->getLayout()->getSubalbum() != NULL)
    window->getLayout()->getSubalbum()->updateButtons(false);
  
  QString albumXML = QFileDialog::getOpenFileName(NULL,
                                                  "XML Files (*.xml)",
                                                  this,
                                                  "open file dialog",
                                                  "Choose an album to load" );

  //if null return
  if(albumXML.isNull())
  {
    //nolonger busy
    setBusy(false);
    window->getLayout()->getSubalbums()->updateButtons(true);
    if(window->getLayout()->getSubalbum() != NULL)
      window->getLayout()->getSubalbum()->updateButtons(true);
    return;
  }
    
  qApp->setOverrideCursor( QCursor(Qt::WaitCursor));
  
  //pop up load dialog
  LoadDialog* dialog = new LoadDialog();
  
  //get size and location of application window
  QRect appRec = qApp->mainWidget()->geometry();

  //center this dialoag within application window
  int x, y;
  if(400 < appRec.width())
  { x = appRec.x() + ((appRec.width() - 400)/2); }
  else
  { x = appRec.x(); }
  if(300 < appRec.height())
  { y = appRec.y() + ((appRec.height() - 300)/2); }
  else
  { y = appRec.y(); }
    
  dialog->setGeometry(x, y, 400, 300);
  dialog->show();
  dialog->repaint();
  
  //create a new album
  dialog->printMessage("Freeing old album");
  delete albm;
  albm = new Album();
  
  //attempt to load xml file
  albm->importFromDisk(dialog, albumXML);

  //refresh screen
  refresh();
  
  window->refresh();
  //close dialog
  delete dialog;
  
  //nolonger busy
  setBusy(false);
  window->getLayout()->getSubalbums()->updateButtons(true);
  if(window->getLayout()->getSubalbum() != NULL)
    window->getLayout()->getSubalbum()->updateButtons(true);
  qApp->restoreOverrideCursor();
}
//==============================================
void TitleWidget::saveAlbum()
{
  //set busy flag and disable buttons
  setBusy(true);
  window->getLayout()->getSubalbums()->updateButtons(false);
  if(window->getLayout()->getSubalbum() != NULL)
    window->getLayout()->getSubalbum()->updateButtons(false);

  //if album not previously saved then
  //run saveas dialog
  if(!getAlbum()->prevSave())
  {
    saveAsAlbum();
  }
  //else bring up save dialog and save
  else
  {
    qApp->setOverrideCursor( QCursor(Qt::WaitCursor));
    
    //pop up save dialog
    SaveDialog* dialog = new SaveDialog();
    
    //get size and location of application window
    QRect appRec = qApp->mainWidget()->geometry();

    //center this dialoag within application window
    int x, y;
    if(400 < appRec.width())
    { x = appRec.x() + ((appRec.width() - 400)/2); }
    else
    { x = appRec.x(); }
    if(300 < appRec.height())
    { y = appRec.y() + ((appRec.height() - 300)/2); }
    else
    { y = appRec.y(); }
    
    dialog->setGeometry(x, y, 400, 300);
    dialog->show();
    dialog->repaint();
  
    //sync current subalbum to ensure all photo descriptions saved
    if(  window->getLayout()->getSubalbum() != NULL )
      window->getLayout()->getSubalbum()->syncPhotos();
    
    getAlbum()->exportToDisk(dialog);
  
    //close dialog
    delete dialog;
    
    qApp->restoreOverrideCursor();
  }
  
  //nolonger busy
  setBusy(false);
  window->getLayout()->getSubalbums()->updateButtons(true);
  if(window->getLayout()->getSubalbum() != NULL)
    window->getLayout()->getSubalbum()->updateButtons(true);
}
//==============================================
void TitleWidget::saveAsAlbum()
{
  //get directory name in which album directory will be placed in
  QString dirName = QFileDialog::getExistingDirectory(NULL,
                                                      this,
                                                      "get existing directory",
                                                      "Choose Directory for Album Folder",
                                                      TRUE);
  //if null return
  if(dirName.isNull())
    return;
    
  //create qdir object for specified directory  
  QDir d(dirName);
  //check if readable
  if(!d.isReadable())
    return;
    
  //create directory for album in this one
  //used album name changing spaces to _'s
  QString albumName = getAlbum()->getName();
  
  //if album name is an invalid (blank or just spaces) return
  if(albumName.stripWhiteSpace() == "")
    return;
  
  albumName.replace( QChar(' '), "_" );
  albumName.replace( "<", "" );
  albumName.replace( ">", "" );
  albumName.replace( "&", "and" );
  albumName.replace( "\"", "" );
  albumName.replace( "\'", "" );
  albumName.replace( "?", "" );
  
  //if unable to create directory return
  if(!d.mkdir(albumName))
  {
    cout << "Error! Unable to create save folder\n";
    return;
  }

  qApp->setOverrideCursor( QCursor(Qt::WaitCursor));
  
  //get path of full directory now
  QString fullPath = d.absPath() + "/" + albumName;
  
  //pop up save dialog
  SaveDialog* dialog = new SaveDialog();
  dialog->setGeometry(200, 200, 400, 300);
  dialog->show();
  dialog->repaint();
  
  //save
  getAlbum()->exportToDisk(dialog, fullPath);
  
  //close dialog
  delete dialog;
  
  qApp->restoreOverrideCursor();
}
//==============================================
void TitleWidget::aboutProgram()
{
  if(about == NULL)
  {
    about = new About();
    connect( about, SIGNAL(aboutClosed()),
             this, SLOT(aboutClosed()));
    about->show();
  
    //get size and location of application window
    QRect appRec = qApp->mainWidget()->geometry();
    QRect aboutRec = about->geometry();
    
    //center this dialoag within application window
    int x, y;
    if(aboutRec.width() < appRec.width())
    { x = appRec.x() + ((appRec.width() - aboutRec.width())/2); }
    else
    { x = appRec.x(); }
    if(aboutRec.height() < appRec.height())
    { y = appRec.y() + ((appRec.height() - aboutRec.height())/2); }
    else
    { y = appRec.y(); }
    
    about->move( QPoint( x, y) );
    
    
  }
  else
  {
    about->raise();   
  }
}
//==============================================
void TitleWidget::aboutClosed()
{
  delete about;
  about = NULL;
}
//==============================================
void TitleWidget::setSetButtonState(bool state)
{
  setImage->setEnabled(state);
}
//==============================================
bool TitleWidget::getBusy()
{
  return busy;
}
//==============================================
void TitleWidget::setBusy(bool val)
{
  busy = val;
  
  //disable/enable file operations
  if(busy)
  {
    file->setItemEnabled(NEW_ALBUM, false);
    file->setItemEnabled(OPEN_ALBUM, false);
    file->setItemEnabled(SAVE_ALBUM, false);
    file->setItemEnabled(SAVEAS_ALBUM, false);
    cacheSetButtonState = setImage->isEnabled();
    setSetButtonState(false);
  }
  else
  {
    file->setItemEnabled(NEW_ALBUM, true);
    file->setItemEnabled(OPEN_ALBUM, true);
    file->setItemEnabled(SAVE_ALBUM, true);
    file->setItemEnabled(SAVEAS_ALBUM, true);
    setSetButtonState(cacheSetButtonState);
  }
}
//==============================================
